#!/usr/bin/env python
#-*- coding: iso-8859-15 -*-

#################################################################################
## Coban - Module de calcul de chute de tension                                ## 
## Copyright (C) 2008  Stphan GURIN                                          ##
## contact : setthe@tuxfamily.org                                              ##
##                                                                             ##
## This program is free software; you can redistribute it and/or               ##
## modify it under the terms of the GNU General Public License                 ##
## as published by the Free Software Foundation; either version 2              ##
## of the License, or (at your option) any later version.                      ##
##                                                                             ##
## This program is distributed in the hope that it will be useful,             ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of              ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               ##
## GNU General Public License for more details.                                ##
##                                                                             ##
## You should have received a copy of the GNU General Public License           ##
## along with this program; if not, write to the Free Software                 ##
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. ##
#################################################################################

import wx

############################################################################
##                                                                        ##
##                       Module chute de tension                          ##
##                          Version 0.1.2                                 ##
##                                                                        ##
##                                                                        ##
##                                                                        ##
############################################################################

version = 'Chute de tension 0.1.2'
sectCu = ' 1.5 2.5 4 6 10 16 25  35 50 70 95 120 150 185 240 300 400 500 630'.split()
sectAlu = '25 35 50 70 95 120 150 185 240 300 400 630'.split()
statusText = u"Section de cble"


class OngletChuteInv(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        self.parent = parent
        
        
        #-------------- cration du sizer principal -------------------
        
        self.gbs = wx.GridBagSizer(hgap=5, vgap=5)

        #--------------- Cration des widgets -------------------------------
        
        # choix des mtaux
        liste1 = ['Cuivre', 'Aluminium']
        self.metal = wx.RadioBox(self, -1, u' Type de cble ', (-1, -1), wx.DefaultSize, 
                    liste1, 1, style = wx.RA_SPECIFY_COLS)
        self.metal.SetToolTipString(u'Choix du mtal des conducteurs')
        
        # Choix tension
        liste2 = [u'240V monophas', u'400V triphas cos Phi = 0,8']
        self.tension = wx.RadioBox(self, -1, u' Tension ', (-1, -1), wx.DefaultSize,
                    liste2, 1, wx.RA_SPECIFY_COLS)
        self.tension.SetToolTipString(u"Choix de la tension d'alimentation")
        
        # Chute de tension
        self.labelChute = wx.StaticText(self, -1, 'Chute de tension (en %) :')
        self.chute = wx.SpinCtrl(self, -1, "", size=(50,-1))
        self.chute.SetRange(1,9)
        self.chute.SetValue(3)
        
        # Saisie de la longueur de cable
        self.labelLg = wx.StaticText(self, -1, u'Longueur du cble (en mtres) :')
        self.lg = wx.SpinCtrl(self, -1, "", size=(50,-1))
        self.lg.SetRange(1, 999)
        self.lg.SetValue(100)
        
        # Saisie de l'intensit
        self.labelI = wx.StaticText(self, -1, u'Intensit (en Ampres) :')    
        self.intensite = wx.SpinCtrl(self, -1, "", size=(50,-1))
        self.intensite.SetRange(1,650)
        self.intensite.SetValue(50)
        
        # sparation
        sep = wx.BoxSizer(wx.VERTICAL)
        sep.Add(wx.StaticLine(self), 0, wx.EXPAND|wx.ALL, 5)
        
        # Fentre des rsultats
        self.labelResult = wx.StaticText(self, -1, u"Section calcule :", style=wx.ALIGN_RIGHT)
        self.Result = wx.StaticText(self, -1, '... mm2')
        self.Result.SetForegroundColour('red')
        self.Result.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD))
        
        # Fentre sections prconises
        self.labelPreco = wx.StaticText(self, -1, u"Section normalise :", style=wx.ALIGN_RIGHT)
        self.preco = wx.StaticText(self, -1, '... mm2')
        self.preco.SetForegroundColour('blue')
        self.preco.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD))
        
        #---------------- Binds ---------------------------
        
        self.Bind(wx.EVT_RADIOBOX, self.onCalcul, self.metal)        
        self.Bind(wx.EVT_RADIOBOX, self.onChute, self.tension)
        self.Bind(wx.EVT_SPINCTRL, self.onCalcul, self.chute)
        self.Bind(wx.EVT_SPINCTRL, self.onCalcul, self.lg)
        self.Bind(wx.EVT_SPINCTRL, self.onCalcul, self.intensite)

        
        #--------------- Ajout des wigets au sizer ---------------------
        
        self.gbs.Add(self.metal, (0, 0))
        self.gbs.Add(self.tension, (0, 1))
        self.gbs.Add(self.labelChute, (1, 0))
        self.gbs.Add(self.chute, (1, 1))
        self.gbs.Add(self.labelI, (2, 0))
        self.gbs.Add(self.intensite, (2, 1))
        self.gbs.Add(self.labelLg, (3,0))
        self.gbs.Add(self.lg, (3,1))
        self.gbs.Add(sep, (4,0), (1,4), wx.EXPAND)
        self.gbs.Add(self.labelResult, (5,0))
        self.gbs.Add(self.Result, (5,1))
        self.gbs.Add(self.labelPreco, (6,0))
        self.gbs.Add(self.preco, (6,1))
        
        self.SetSizerAndFit(self.gbs)
        self.SetClientSize(self.GetSize())
        
    def onChute(self, evt):
        tens = self.tension.GetStringSelection()
        if tens == u'240V monophas':
            self.chute.SetValue(3)
        else :
            self.chute.SetValue(5)
        self.onCalcul(evt)
        
    def onCalcul(self, evt):
        
        # choix de ro selon le type de cble
        m = self.metal.GetStringSelection()
        if m == 'Cuivre':
            ro = 23
            sect = sectCu
        else : 
            ro = 37
            sect = sectAlu
        
        # choix de la chute de tension en %
        deltau = self.chute.GetValue()
        if deltau == 0:
            deltau = 3
        
        # choix de la chute de tension en Volts, du coef b, de lbda
        v = self.tension.GetStringSelection()
        if v == u'240V monophas':
            u = deltau*2400
            b = 2
            lbda = 0.13
            cosphi = 1
            sinphi = 0
        else :         
            u = deltau*2400
            b = 1
            lbda = 0.08
            cosphi = 0.8
            sinphi = 0.6
                
        # choix intensit
        i = self.intensite.GetValue()
        if i == 0:
            i = 50
        
        # choix longueur de cble
        l = self.lg.GetValue()
        if l == 0:
            l = 100
             
        # calcul de la section s
        s = (ro*l*cosphi)/((u/(b*i))-(lbda*l*sinphi))
        s = round (s, 2)
        
        if s >= 631 or s < 0 :
            self.Result.SetLabel(u'section non calcule')
            self.preco.SetLabel(u'section non dfinie')
        else :
            # affiche la section calcule
            self.Result.SetLabel('%s mm2' %s)
            # choix de la section normalise
            a = 0            
            sectPreco = sect[a]
            for sectNorm in sect :
                if float(sectNorm) < s:
                    sectPreco = sect[a+1]
                a += 1
            # affiche la section normalise surieure  la section calcule
            self.preco.SetLabel('%s mm2' %sectPreco)
        


            
            
            
            
class OngletChute(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        self.parent = parent
        
        
        #---------- cration du sizer principal ----------------
        
        self.gbs = wx.GridBagSizer(hgap=5, vgap=5)

        #------------- Cration des widgets ---------------------
        # Mono / Tri
        listeCircuit=[u'monophas', u'triphas']
        labelMonoTri = wx.StaticText(self, -1, u'Type de circuit')
        self.monoTri = wx.Choice(self, -1, choices=listeCircuit)
        self.monoTri.SetSelection(1)
        self.monoTri.SetFocus()
        self.labelCosPhi = wx.StaticText(self, -1, u'cos Phi = 0,8')
        # type de cble :
        listeCable = ['Cuivre', 'Aluminium']
        labelCable = wx.StaticText(self, -1, u"Type de cble")
        self.cable = wx.Choice(self, -1, choices=listeCable)
        self.cable.SetSelection(0)
        # Longueur du cble
        labelLg = wx.StaticText(self, -1, u'Longueur du cble\n(en mtres) :')
        self.lg = wx.SpinCtrl(self, -1, "", size=self.cable.GetSize())
        self.lg.SetRange(1, 999)
        self.lg.SetValue(100)
        # Section du conducteur
        labelSection = wx.StaticText(self, -1, u'Section du conducteur\n (en mm)')
        self.section = wx.Choice(self, -1, choices=sectCu, size=self.cable.GetSize())
        self.section.SetSelection(5)
        # Courant d'emploi
        labelCourant = wx.StaticText(self, -1, u"Courant d'emploi\n(en Ampres)")
        self.courant = wx.SpinCtrl(self, -1, "", size=self.cable.GetSize())
        self.courant.SetRange(1,999)
        self.courant.SetValue(100)
        # Tension Ph/N
        labelTension = wx.StaticText(self, -1, u'Tension entre phase\net neutre (en Volts)')
        listeTension = ['110V', '230V', '240 V', '380 V', '400 V']
        self.tension = wx.Choice(self, -1, choices=listeTension, size=self.cable.GetSize())
        self.tension.SetSelection(1)
        # Bouton calcul
        self.btnCalc = wx.Button(self, -1, 'Calculer')
        self.btnCalc.SetBackgroundColour('yellow')
        self.btnCalc.SetDefault()
        # Sparation
        sep = wx.BoxSizer(wx.VERTICAL)
        sep.Add(wx.StaticLine(self), 0, wx.EXPAND|wx.ALL, 5)
        # Rsultats
        labelResult = wx.StaticText(self, -1, u'Rsultats')
        labelResult.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD))
        labelChuteDelta=wx.StaticText(self, -1, u'Chute de tension\n(en %)')
        self.labelDeltaU = wx.StaticText(self, -1, u'... %')
        self.labelDeltaU.SetForegroundColour('blue')
        self.labelDeltaU.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD))
        labelChuteV = wx.StaticText(self, -1, u'Chute de tension\n(en Volts)')
        self.labelChuteVolt = wx.StaticText(self, -1, u'... V')
        self.labelChuteVolt.SetForegroundColour('blue')
        self.labelChuteVolt.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD))
        
        # binds
        self.Bind(wx.EVT_CHOICE, self.onRaz)
        self.Bind(wx.EVT_SPINCTRL, self.onRaz)
        self.Bind(wx.EVT_BUTTON, self.onCalcul, self.btnCalc)
        
        # placement des widgets
        self.gbs.Add(labelMonoTri, (0,0))
        self.gbs.Add(self.monoTri, (0,1))
        self.gbs.Add(self.labelCosPhi, (0,2))
        self.gbs.Add(labelCable, (1,0))
        self.gbs.Add(self.cable, (1,1))
        self.gbs.Add(labelLg, (2,0))
        self.gbs.Add(self.lg, (2,1))
        self.gbs.Add(labelSection, (3,0))
        self.gbs.Add(self.section, (3,1))
        self.gbs.Add(labelCourant, (4,0))
        self.gbs.Add(self.courant, (4,1))
        self.gbs.Add(labelTension, (5,0))
        self.gbs.Add(self.tension, (5,1))
        self.gbs.Add(self.btnCalc, (6,1))
        self.gbs.Add(sep, (7,0), (1,4),wx.EXPAND)
        self.gbs.Add(labelResult, (8,0))
        self.gbs.Add(labelChuteDelta, (9,0))
        self.gbs.Add(self.labelDeltaU, (9,1))
        self.gbs.Add(labelChuteV, (10,0))
        self.gbs.Add(self.labelChuteVolt, (10,1))
        
        # taille du sizer
        self.SetSizerAndFit(self.gbs)
        self.SetClientSize(self.GetSize())
        
    def onRaz(self, evt):
        self.labelChuteVolt.SetLabel('... V')
        self.labelDeltaU.SetLabel('... %')
        if self.monoTri.GetSelection() == 0:
            self.labelCosPhi.SetLabel(u'')
        else :
            self.labelCosPhi.SetLabel(u'cos Phi = 0,8')
        
    def onCalcul(self, evt):
        # import si mono ou tri
        if self.monoTri.GetSelection()==0:
            b = 2
            cosPhi = 1
            sinPhi = 1
        else : 
            b = 1
            cosPhi = 0.8
            sinPhi = 0.6
        # import de la rsistivit
        if self.cable.GetSelection() == 0:
            rau1 = 23
        else :
            rau1 = 37
        # import de la longueur
        L = self.lg.GetValue()
        # import de la section
        S = float(self.section.GetStringSelection())
        # import du courant d'emploi
        Ib = self.courant.GetValue()
        # import de la tension
        if self.tension.GetSelection() == 0:
            Uo = 110
        if self.tension.GetSelection() == 1:
            Uo = 230
        elif self.tension.GetSelection() == 2:
            Uo = 240
        elif self.tension.GetSelection() == 3:
            Uo = 380
        else :
            Uo = 400
        # coef lambda
        Lambda = 0.08
        
        # rsultats
        u = round((b*(rau1*(L/S)*cosPhi+Lambda*L*sinPhi)*Ib)/1000, 3)
        deltaU = round((100*(u/Uo)), 2)
        self.labelChuteVolt.SetLabel(u'%s V'%u)
        self.labelDeltaU.SetLabel((u'%s '%deltaU)+'%')
        
          
        
        
        
        

class ModuleChute(wx.Frame):
    """Calcul des sections de cble"""
    def __init__(self, parent):
        wx.Frame.__init__(self, parent, -1, title = version,
                      style=wx.DEFAULT_FRAME_STYLE 
                      ^(wx.RESIZE_BORDER | wx.MAXIMIZE_BOX))
        self.parent = parent
        
        #------- Cration du panel -----------------------        
        panel =  wx.Panel(self)
        
        #------- Cration du sizer principal ---------------------
        sizer = wx.GridBagSizer(hgap=5, vgap=5)
        
        #------- Cration de statusbar -----------------        
        self.CreateStatusBar()
        self.SetStatusText(statusText)
        
        #-------- Ajout du notebook au panel -------------
        self.nb = wx.Notebook(panel)
        
        #-------- Cration des onglets -------------------
        self.page1 = OngletChuteInv(self.nb)
        self.page2 = OngletChute(self.nb)
        
        #-------- Ajout des onglets au notebook -----------
        self.nb.AddPage(self.page1, u"calcul invers")
        self.nb.AddPage(self.page2, u"Chute de tension")
        self.nb.ChangeSelection(1)
        
        #-------- Bouton quitter ---------------------------
        btnQuit = wx.Button(panel, wx.ID_CLOSE, "Quitter")
        self.Bind(wx.EVT_BUTTON, self.onQuitter, btnQuit)
        
        #-------- Binds -----------------------------------
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.chgOnglet, self.nb)
        self.Bind(wx.EVT_CLOSE, self.onQuitter)
        
        #-------- Ajout des Widgets au sizer ------------------
        sizer.Add(self.nb, (0,0),border=2)
        sizer.Add(btnQuit, (1,0))
        
        #-------- Ajout du sizer au panel ---------------------------
        panel.SetSizerAndFit(sizer)
        self.SetClientSize(panel.GetSize())
        
        self.Bind(wx.EVT_CLOSE, self.onQuitter)
        
    def onQuitter(self, evt):
        """Quitter l'application"""
        self.parent.btnChute.Enable(True)
        self.parent.menuModuleChute.Enable(True)
        self.Destroy()
        
    def chgOnglet(self, evt):
        # mettre une aide en foncion de l'onglet dans la statusbar
        numOnglet = self.nb.GetSelection()
        if numOnglet == 0 :
            self.SetStatusText(statusText)
        if numOnglet == 1 :
            self.SetStatusText(u"Chute de tension")
        
    